import cleanDeep from 'clean-deep';
import { diff } from 'deep-object-diff';
import isPlainObject$1 from 'is-plain-obj';

/*
 * ts-omit-deep.ts
 *
 * based on https://github.com/jonschlinkert/omit-deep/blob/master/index.js
 *
 * Created by Dr. Maximillian Dornseif 2023-10-06 in huWaWi3 33.1.0
 * Copyright (c) 2023 Dr. Maximillian Dornseif
 */
function omitDeep(value, keys) {
  keys = [keys].flat();
  if (typeof value === 'undefined') {
    return {};
  }
  if (Array.isArray(value)) {
    for (var i = 0; i < value.length; i++) {
      try {
        value[i] = omitDeep(value[i], [keys].flat());
      } catch (e) {
        console.warn("Zugriff auf value[" + i + "] hat nicht geklappt", value);
        console.error(e);
      }
    }
    return value;
  }
  if (!isPlainObject$1(value)) {
    return value;
  }
  for (var j = 0; j < keys.length; j++) {
    try {
      delete value[keys[j]];
    } catch (e) {
      console.warn("L\xF6schen von " + keys[j] + " hat nicht geklappt", value);
      console.error(e);
    }
  }
  for (var key in value) {
    if (value.hasOwnProperty(key)) {
      try {
        value[key] = omitDeep(value[key], keys);
      } catch (e) {
        console.warn("Zugriff auf value[" + key + "] hat nicht geklappt", value);
        console.error(e);
      }
    }
  }
  return value;
}

/*
 * ts-omit-deep-underscore.ts
 *
 * based on https://github.com/jonschlinkert/omit-deep/blob/master/index.js
 *
 * Created by Dr. Maximillian Dornseif 2023-10-06 in huWaWi3 33.1.0
 * Copyright (c) 2023 Dr. Maximillian Dornseif
 */
/** Omit all properties starting with an underscore */
function omitDeepUnderscore(value) {
  if (typeof value === 'undefined') {
    return {};
  }
  if (Array.isArray(value)) {
    for (var i = 0; i < value.length; i++) {
      value[i] = omitDeepUnderscore(value[i]);
    }
    return value;
  }
  if (!isPlainObject$1(value)) {
    return value;
  }
  var unwantedKeys = Object.keys(value).filter(function (s) {
    return s.startsWith('_');
  });
  for (var j = 0; j < unwantedKeys.length; j++) {
    delete value[unwantedKeys[j]];
  }
  for (var key in value) {
    if (value.hasOwnProperty(key)) {
      value[key] = omitDeepUnderscore(value[key]);
    }
  }
  return value;
}

// diese Properties wollen wir nicht im Editor
var OMIT_PROPERTIES = ['__typename', 'id', 'updated_at', 'created_at'];
function cleanGqlInput(entity, omit) {
  if (omit === void 0) {
    omit = OMIT_PROPERTIES;
  }
  return sortObject(
  // cleanDeep( Removes empty objects, arrays, empty strings, NaN, null and undefined values from objects.
  omitDeepUnderscore(omitDeep(
  // removes unwanted properties
  entity, omit)));
}
function cleanDiff(originalObj, updatedObj) {
  // We have an issue here with https://github.com/mattphillips/deep-object-diff/issues/14
  // https://github.com/mattphillips/deep-object-diff/issues/79
  // https://github.com/mattphillips/deep-object-diff/issues/94
  return cleanDeep(diff(cleanGqlInput(originalObj), cleanGqlInput(updatedObj)), {
    emptyArrays: true,
    emptyObjects: true,
    emptyStrings: false
  });
}
// from https://github.com/IndigoUnited/js-deep-sort-object/blob/master/index.js
function isPlainObject(obj) {
  return Object.prototype.toString.call(obj) === '[object Object]';
}
function defaultSortFn(a, b) {
  return a.localeCompare(b);
}
function sortObject(src, comparator) {
  if (Array.isArray(src)) {
    src.map(function (item) {
      return sortObject(item, comparator);
    });
  }
  if (isPlainObject(src)) {
    var out = {};
    Object.keys(src).sort(comparator || defaultSortFn).forEach(function (key) {
      out[key] = sortObject(src[key], comparator);
    });
    return out;
  }
  return src;
}

export { OMIT_PROPERTIES, cleanDiff, cleanGqlInput, omitDeep, omitDeepUnderscore, sortObject };
//# sourceMappingURL=graphql-clean-diff.esm.js.map
